class ProposalReason < ActiveRecord::Base
  belongs_to :action, class_name: ProposalAction, foreign_key: :proposal_action_id, inverse_of: :reasons
  
  def log
    puts reason.gsub(/<.*?>/,'') + " / #{(action.short_action2 || action.short_action).to_s} / #{score}%" 
    self
  end
  
  ### heuristic -- guessing reasons and creating ProposalReasons 
  
  def self.guess_missing_junction_guideposts
    @category = 'add missing junction guideposts'
    category_cleanup(@category)
    
    KctGuidepost.all.each do |kct_guidepost|
      next if kct_guidepost.exists? or (kct_guidepost.kct_trails.size<2)
      kct_trailA = kct_guidepost.kct_trails.first
      kct_trailB = kct_guidepost.kct_trails.last
      next if (kct_trailA.kct_guideposts & kct_trailB.kct_guideposts).size != 1
      
      osm_trailA = kct_trailA.osm_trail
      osm_trailB = kct_trailB.osm_trail
      next if osm_trailA.nil? or osm_trailB.nil? # dál jen pokud jsou trasy v OSM
      
      intersection = osm_trailA.gpx.flat_coordinates_array & osm_trailB.gpx.flat_coordinates_array
      next if intersection.size != 1 # dál jen pokud je průnik jednobodový
      intersection = intersection.first
      next if !OsmGuidepost.nearby(intersection, 200).empty? # jen pokud není do 200m žádný rozcestník
      
      ProposalReason.new(
        category: @category,
        reason: "Trasy #{kct_trailA.kctref} a #{kct_trailB.kctref} se se stýkají v jediném bodě u kterého by měl být dle KČT rozcestník #{kct_guidepost.kctref}",
        action: ProposalAction.update_or_create_by(catid: "cgj|#{kct_guidepost.kctref}|#{intersection[0]}|#{intersection[1]}"),
        score: 100
      ).log.save
    end
  end
  
  def self.guess_kctrefs_of_osm_trails  
    category_cleanup(category = 'guess kctref of osm trail by near osm_guideposts')
    @near = 30 # jak blizko je blizko v metrech
    index = 0
    
    OsmTrail.all.each do |osm_trail|
      index = index + 1
      next if !osm_trail.trail.nil?
      gpx = osm_trail.gpx
      next if gpx.flat_coordinates_array.empty?
      
      # vypocteni obdelniku ktery obsahyje osm_trail
      lat_min,lat_max =  gpx.flat_coordinates_array.collect{ |p| p[0] }.minmax
      lon_min,lon_max =  gpx.flat_coordinates_array.collect{ |p| p[1] }.minmax
      lat_min = lat_min-0.009/1000*@near; lat_max = lat_max+0.009/1000*@near
      lon_min = lon_min-0.0136/1000*@near; lon_max = lon_max+0.0136/1000*@near
      
      osm_guideposts = OsmGuidepost.where(lat: lat_min .. lat_max, lon: lon_min..lon_max)
      osm_guideposts = osm_guideposts.select{ |og| gpx.distance_to(og.coordinates) < @near }
      kct_guideposts = osm_guideposts.collect{ |og| 
        if og.guidepost.nil?
          KctGuidepost.find_by_nazev_normalized(og.nazev_normalized)
        else
          og.guidepost.kct_guidepost
        end
      }.compact
      
      all_kct_trails_repeated = kct_guideposts.collect{ |kg| kg.kct_trails }.flatten.sort_by{ |kt| kt.kctref }
      all_free_kct_trails_with_multiplicity = all_kct_trails_repeated.chunk{|y| y}.map{|y, ys| [y, ys.length]}.sort_by{|y| y[1]}.select{|y| y[0].trail.nil? }
      # it will contain something like [[KctTrailA,5],[KctTrailB,4],...]
      
      next if all_free_kct_trails_with_multiplicity.empty?
      kct_trail = all_free_kct_trails_with_multiplicity.first[0]
      multiplicity = all_free_kct_trails_with_multiplicity.first[1]
      next if kct_trail.barva != osm_trail.barva
      
      if multiplicity > 2 && !kct_trail.andand.trail.andand.osm_trail.nil?
        cat = kct_trail.trail.nil? ? 'jkot-nt' : 'jkot-et'  # existing trail
        ProposalReason.new(
          category: category,
          reason: linkify("Neidentifikovaná trasa #{osm_trail} má blízko #{multiplicity}ks rozcestníků z trasy #{kct_trail} a je stejné barvy"),
          action: ActionJoinKctOsmTrail.update_or_create_by(cat, kct_trail.kctref, osm_trail.osmid),
          score: multiplicity*30 - (multiplicity<4?10:0)  # ie (20,50,)80,120,150,180,...
        ).log.save
      end
      
    end
  end
  
  def self.guess_branching_points
    @category = 'guess branching points on trails'
    category_cleanup(@category)
  end
  
  def self.guess_sign_by_osm_and_kct_signs 
    category_cleanup(category = 'guess signs by osm and kct')
    Guidepost.all.each do |guidepost|
      next if guidepost.osm_guidepost.nil?
      missing_osm_trails = guidepost.osm_guidepost.osm_trails - guidepost.trails.collect(&:osm_trail)
      missing_osm_trails.each do |osm_trail|  # na této osm_trail je guidepost podle OSM ale ne podle revdbf
        next if guidepost.kct_guidepost.nil?
        if guidepost.kct_guidepost.kct_trails.include? osm_trail.kct_trail # pokud je to tak i podle KCT
          trail = osm_trail.trail
          raise 'Nemelo by to tu byt' if guidepost.trails.include? trail
          ProposalReason.new(
            category: category,
            reason: linkify("Rozcestník rg:#{guidepost.kctref} je na trase ot:#{osm_trail.osmid} i na kt:#{osm_trail.kct_trail.kctref}, takže bude i na rt:#{trail.kctref}"),
            action: ActionInsertGuidepostToTrail.update_or_create_by('igtot',guidepost.id,trail.id),
            score: 120,
          ).log.save
        end
      end
    end
  end
  
  def self.guess_wrong_osm_guideposts_of_osm_trails_by_distance 
    category_cleanup(category = 'distant osm_guideposts')
    OsmTrail.all.each do |osm_trail|
      osm_trail.osm_guideposts.each do |og|
        dist = osm_trail.distance_to_including_excursions(og)      # kolk metrů už je moc že to není na trase
        
        score = case dist
        when 100..200 then 80
        when 200..300 then 90
        when 300..500 then 110
        when 500..900 then 130
        when 900..3000 then 150
        when 3000..Float::INFINITY then 200
        else nil
        end
        next if score.nil?

        ProposalReason.new(
          category: category,
          reason: linkify("OSM rozcestník og:#{og.osmid} je daleko od své osm trasy ot:#{osm_trail.osmid}"),
          action: ProposalAction.update_or_create_by(catid: "dogfot|#{osm_trail.osmid}|#{og.osmid}"),
          score: score
        ).log.save
      end
    end
  end
  
  def self.guess_very_small_gaps_and_diversions 
    category_cleanup(category = 'very small gaps')
    @kolik_je_malo_v_metrech_od = 0
    @kolik_je_malo_v_metrech_do = 50
    @ignore_list = [ 'vsg|1559824|[50.6335264, 13.6611251]', 'vsg|2074951|[49.3211975, 13.7009907]', 'vsg|1524902|[49.8990372, 16.437087]', 'vsg|1067891|[49.3208318, 13.7003092]']
    
    OsmTrail.select{ |ot| ot.typ!='naučná'}.each do |osm_trail|
      osm_trail.gaps.each do |gap|
        dist = Map.distance(*gap)
        if (@kolik_je_malo_v_metrech_od<dist) and (dist<@kolik_je_malo_v_metrech_do)
          bp = osm_trail.gpx.branching_points
          if bp.include?(gap.first) or bp.include?(gap.last)
            mess1 = 'velmi malá odbočka'; mess3='vsd';
            next
          else
            mess1 = 'velmi malá mezera'; mess3='vsg';
          end
          catid = "#{mess3}|#{osm_trail.osmid}|#{gap}"
          next if @ignore_list.include?(catid)
          
          ProposalReason.new(
            category: category,
            reason: linkify("Na osm trase ot:#{osm_trail.osmid} (<a href=\"/osm_trails/osmid-#{osm_trail.osmid}/reimport\">reimport</a>) " +
            "je #{mess1} délky #{dist} metrů."),
            action: ProposalAction.update_or_create_by(catid: catid),
            score: 100 + case dist # malé mezery mají větší skóre
            when 0..1 then 10
            when 1..5 then 9
            when 5..10 then 8
            when 10..20 then 7
            when 20..40 then 6
            when 40..80 then 5
            when 80..200 then 4
            end
          ).log.save
        end
      end
    end
  end
  
  def self.guess_identical_trails_by_unique_color_around_guidepost 
    category_cleanup(category = 'identical coloured trails around guidepost')
    Guidepost.all.each do |guidepost|
      next if guidepost.kct_guidepost.nil?
      next if guidepost.osm_guidepost.nil?
      puts "Zkoumám guidepost #{guidepost.kct_guidepost.nazev}/#{guidepost.osm_guidepost.nazev}"
      
      Map.barvy.each do |barva|
        kct_possible_trails =  guidepost.kct_guidepost.kct_trails.select { |kt| kt.barva==barva }
        osm_possible_trails =  guidepost.osm_guidepost.osm_trails.select { |kt| kt.barva==barva }
        
        if kct_possible_trails.size == 1 and  osm_possible_trails.size==1
          puts "Barva #{barva} je jednozačná"
          kt = kct_possible_trails.first
          ot = osm_possible_trails.first
          
          if !ot.trail.nil?
            if (ot.trail == kt.trail)
              puts "ale už přiřazená"; next
            else
              puts "Podivný výsledek - OSM trail má už trail, ale ten se neshoduje s KČT trail"; next
            end
          else
            reason = linkify("Od rozcestníku rg:#{guidepost.id} vede jen jedna #{barva} značka "+
              "jak v KČT(kt:#{kt.kctref}) tak v OSM (ot:#{ot.osmid})")
            cat = kt.trail.nil? ? 'jkot-nt' : 'jkot-et'  # existing trail
            ProposalReason.new(
              category: category, reason: reason,
              action: ActionJoinKctOsmTrail.update_or_create_by(cat, kt.kctref, ot.osmid),
              score: (kt.kct_guideposts.size > 10) ? 30 : ( (kt.kct_guideposts.size > 4 ) ? 40 : 45  )  # čím míň má rozcestníků trasa, tím je jednotlivá informace důležitější
            ).log.save
          end          
        else
          puts "Barva #{barva} není jednozačná (#{kct_possible_trails.size}/#{osm_possible_trails.size})"
        end
      end
    end
  end
    
  def self.guess_by_kctref_on_osmtrails 
    category_cleanup(category = 'kctref on osm trails')
    
    OsmTrail.all.each do |osm_trail|
      osmid = osm_trail.osmid
      next unless osm_trail.trail.nil?
      kctref = osm_trail.kctref
      next if kctref.nil?
      next unless kctref.match(/^\d\d\d\d$/) # zatím bereme jen jasné číselné reference
      trail = Trail.find_by_kctref(kctref)
      next if KctTrail.find_by_kctref(kctref).nil?
      
      cat = trail.nil? ? 'jkot-nt' : 'jkot-et' # existing trail
      ProposalReason.new(  
        category: category,
        reason: linkify("OSM trail ot:#{osmid} má kctref #{osm_trail.kctref} "+
          "takže asi patří ke KČT trase kt:#{kctref}"),
          action: ActionJoinKctOsmTrail.update_or_create_by(cat, kctref, osmid),
          score: 65
      ).log.save
    end
  end
      
  def self.guess_by_name_simillarity 
    category_cleanup(category = 'podobnost názvů OSM a KČT')
    
    OsmGuidepost.all.each do |osm_guidepost|
      kctref = osm_guidepost.kctref
      osmid = osm_guidepost.osmid
      
      # otestovat zda už je přiřazen nebo Guidepost existuje
      next if self.bad_kctref?(kctref)
      next unless osm_guidepost.guidepost.nil? 
      next if Guidepost.find_by_kctref(kctref)
      next if (kct_guidepost = KctGuidepost.find_by_kctref(kctref)).nil?
      next if (osm_nazev = osm_guidepost.nazev).nil?
      next if (kct_nazev = kct_guidepost.nazev).nil?
      
      score,reason = self.name_merge_score_and_reason(osm_nazev,kct_nazev)
      next if score<=1
      
      # z toho plyne důvod a akce
      ProposalReason.new(
        category: category,
        reason: "Názvy <a href=\"/osm_guideposts/#{osm_guidepost.id}\">OSM rozcestníku #{osmid}</a> a <a href=\"/kct_guideposts/kctref-#{kctref}\">#{kctref}</a> (tj. #{osm_nazev} / #{kct_nazev}) #{reason}",
        action: ActionJoinOsmGuidepostByKctref.update_or_create_by('jgog-ng', kctref, osmid),
        score: score
      ).log.save
    end    
  end
      
  def self.add_guidepost_by_sez_osm_simillarity 
    category_cleanup(category = 'vytěžení SEZ blízých k OSM')  # also contains proximity check
    
    OsmGuidepost.all.each do |osm_guidepost|
      guidepost = osm_guidepost.guidepost
      next if !guidepost.nil? and !guidepost.sez_guidepost.nil?
      
      sez_guidepost = osm_guidepost.unambiguous_sez_guidepost
      
      next if sez_guidepost.nil?
      next if (osm_nazev = osm_guidepost.nazev).nil?
      next if (sez_nazev = sez_guidepost.nazev).nil?
      next if !guidepost.andand.nazev.blank?
      score,reason = self.name_merge_score_and_reason(osm_nazev,sez_nazev)
      next if score<=1
      
      action = ActionJoinSeznamAndOsmGuidepost.update_or_create_by('jsog', osm_guidepost.osmid, sez_guidepost.id)
      ProposalReason.new(
        category: category, action: action, score: score,
        reason: linkify("Názvy \"#{osm_nazev}\" a \"#{sez_nazev}\" #{reason}"),
      ).log.save
      
      dist = osm_guidepost.distance_to(sez_guidepost).to_i
      dist_score = case dist # za geografickou blizkost
        when 0..5 then 81
        when 5..20 then 70
        when 20..40 then 60
        when 40..60 then 40
        when 60..80 then 0
        when 80..150 then -20
        else -50
      end

      ProposalReason.new(
        category: category, action: action, score: dist_score,
        reason: linkify("Rozcestníky jsou blízko (#{dist}m)")
      ).log.save
      
    end    
  end
      
  def self.guess_by_kctref_on_osmguideposts 
    category_cleanup(category = 'kctref on osm guideposts')
    
    OsmGuidepost.all.each do |osm_guidepost|
      kctref = osm_guidepost.kctref
      osmid = osm_guidepost.osmid
      
      # otestovat zda už je přiřazen
      next if self.bad_kctref?(kctref)
      next unless osm_guidepost.guidepost.nil? 
      
      # kouknout se zda guidepost existuje
      guidepost = Guidepost.find_by_kctref(kctref)
      next unless guidepost.andand.osm_guidepost.nil?
      
      # z toho plyne důvod a akce
      if guidepost.nil?
        catidbase = 'jgog-ng'
        reason = "OSM rozcestník og:#{osmid} má kctref #{osm_guidepost.kctref} a ten není v revizní databázi"
      else # existing guidepost
        catidbase = 'jgog-eg'
        reason = "OSM rozcestník og:#{osmid} má kctref #{osm_guidepost.kctref} takže nejspíš patří k rozcestníku rg:#{kctref}"
      end  

      ProposalReason.new(
        category: category,
        reason: linkify(reason),
        action: ActionJoinOsmGuidepostByKctref.update_or_create_by(catidbase, kctref, osmid),
        score: 70
      ).log.save
    end
  end
  
  def self.guess_name 
    category_cleanup(category = 'name guidepost')
    
    Guidepost.all.each do |guidepost|
      kct_nazev = guidepost.kct_guidepost.andand.nazev
      osm_nazev = guidepost.osm_guidepost.andand.nazev
      
      next if guidepost.osm_guidepost.nil?
      next unless guidepost.nazev.blank?  # je už Guidepost pojmenován
      score,reason = self.name_merge_score_and_reason(osm_nazev,kct_nazev)
      next if score<=65
      
      # z toho plyne důvod a akce
      ProposalReason.new(
        category: category,
        reason: "Názvy og:#{guidepost.osm_guidepost.osmid} a kg:#{guidepost.kct_guidepost.andand.kctref} (tj. #{osm_nazev} / #{kct_nazev}) #{reason}",
        action: ActionSetGuidepostName.update_or_create_by('ng', guidepost.id, osm_nazev),
        score: score+40
      ).log.save
    end
  end
  
  def self.guess_location 
    category_cleanup(category = 'localize guidepost')
    
    Guidepost.all.each do |guidepost|
      next unless guidepost.lat.blank? and guidepost.lon.blank? # Guidepost už je lokalizován 
      og = guidepost.osm_guidepost
      next if og.nil?
      
      ProposalReason.new(  # z toho plyne důvod a akce
        category: category,
        reason: "Rozcestník rg:#{guidepost.kctref} nemá polohu, ale připojený og:#{og.osmid} ano",
        action: ActionSetGuidepostLocation.update_or_create_by("lgfog", guidepost.id, og.osmid),
        score: 105
      ).log.save
    end
  end
  
  def self.changes_from_trails_to_osm_trails 
    category_cleanup(category = 'changes to osm_trail')
    
    Trail.where("kctref LIKE '3%'").all.each do |trail|
      # missing membership in route relations
      next if trail.osm_trail.nil?
      next if trail.kct_trail.nil?
      
      osm_guideposts = trail.osm_trail.osm_guideposts
      missing_osm_guideposts = trail.guideposts.select{ |guidepost| !osm_guideposts.include?(guidepost.osm_guidepost) }.collect{ |guidepost| guidepost.osm_guidepost }.compact
      next if missing_osm_guideposts.empty?
      guessed_zone = trail.kct_trail.guess_zone
      missing_osm_guideposts = missing_osm_guideposts.first 10  # zkrátit na přidání max 10 rozcestníků najednou
      
      ProposalReason.new(
        category: category,
        reason: linkify("Na rt:#{trail.kctref} jsou rozcestníky, jejichž OSM rozcestníky (#{missing_osm_guideposts.collect{|og| "og:#{og.osmid}"}.join(', ')}) nejsou na ot:#{trail.osm_trail.osmid} "),
        action: ActionInsertOsmGuidepostToOsmTrail.update_or_create_by("iogtot", guessed_zone, trail.osm_trail.andand.osmid, missing_osm_guideposts.collect{|og| og.osmid}.join(',')),
        score: 105,        
      ).log.save
    end
  end
  
  def self.changes_from_guideposts_to_osm_guideposts 
    category_cleanup(category = 'changes to osm_guidepost')
    
    Guidepost.all.each do |guidepost|
      osm_guidepost = guidepost.osm_guidepost
      
      next if osm_guidepost.nil?
      next if guidepost.nazev.nil?
      # missing tags
      gt = guidepost.suggested_osm_tags
      ogt = osm_guidepost.json['tags']
      raise osm_guidepost.inspect if ogt.nil?
      missing_tags = gt.keys - ogt.keys
      next if missing_tags.empty?
      kctref = guidepost.kct_guidepost.andand.kctref
      
      # z toho plyne důvod a akce
      ProposalReason.new(
        category: category,
        reason: linkify("U rg:#{kctref} je více informací (#{missing_tags.join(',')}) než v og:#{osm_guidepost.id}"),
        action: ActionSetOsmGuidepostTags.update_or_create_by("chfgtog", guidepost.id, osm_guidepost.osmid),
        score: 105
      ).log.save
    end
  end
  
  def self.guess_existing_guidepost_for_unattached_osm_guidepost 
    category_cleanup(category = 'unattached osm guideposts')
    
    volne_osm_guideposty = OsmGuidepost
    #     volne_osm_guideposty = volne_osm_guideposty.where("osm_guideposts.kctref LIKE 'PS%'")  # debug
    volne_osm_guideposty = volne_osm_guideposty.select{ |og| og.guidepost.nil? }
    
    volne_osm_guideposty.each do |osm_guidepost|
      osm_guidepost.osm_signs.each do|osm_sign|
        trail = osm_sign.trail
        next if trail.nil?
        volne_guideposty = trail.guideposts.without_osm_guidepost
        volne_guideposty.each do |guidepost|
          sign = Sign.find_by(guidepost: guidepost, trail: trail)   #
          kct_sign = KctKilometraz.find_by(kct_guidepost: guidepost.kct_guidepost, kct_trail: trail.kct_trail)   # osefovat guidepost.kct_guidepost.nil
          score = Sign.merge_score(kct_sign,osm_sign)
          if score > 0
            ProposalReason.new(
              category: category,
              reason: linkify("Nenulové skóre na merge rg:#{guidepost.kctref} a og:#{osm_guidepost.osmid} na trase rt:#{trail.kctref}"),
              action: ActionJoinOsmGuidepostByKctref.update_or_create_by('jgog-eg', guidepost.kctref, osm_guidepost.osmid),
              score: score
            ).log.save
          end
        end
      end
    end
  end
  
  def self.guess_trail_color_from_kct_and_osm 
    category_cleanup(category = 'uncolored trail')
    Trail.where(barva: nil).each do |trail|
      ot = trail.osm_trail
      kt = trail.kct_trail
      next if ot.nil? or kt.nil?
      next if ot.barva.nil?
      next if ot.barva!=kt.barva
      next if ot.typ!=kt.typ
      barva = ot.barva
      typ = ot.typ
      ProposalReason.new(
        category: category,
        reason: linkify("Trasa rt:#{trail.kctref} nemá barvu ale barvy KČT a OSM se shodují (#{barva}/#{typ})"),
        action: ActionSetTrailColor.update_or_create_by('colt', trail.id, barva, typ),
        score: 120
      ).log.save
    end
  end
  
  def self.join_guidepost_to_nearby_unattached_osm_guidepost 
    category_cleanup(category = 'unattached osm guideposts near guideposts')
    Guidepost.without_osm_guidepost.with_position.each do |guidepost|
      nearby_osm_guideposts = OsmGuidepost.nearby(guidepost.coordinates, 100)
      next unless nearby_osm_guideposts.size==1 # jen pokud je do 100m jen jeden rozcestník
      distance,osm_guidepost = nearby_osm_guideposts.first
      score = case distance
        when 0..5 then 70
        when 5..10 then 60
        when 10..20 then 40
        when 20..30 then 20
        when 30..50 then 10
        else 0
      end
      ProposalReason.new(
        category: category,
        reason: linkify("Rozcestník og:#{osm_guidepost.osmid} je blízko rg:#{guidepost.kctref} a je v okolí jediný"),
        action: ActionJoinOsmGuidepostByKctref.update_or_create_by('jgog-eg', guidepost.kctref, osm_guidepost.osmid),
        score: score
      ).log.save
    end
  end
  
  ### pomocne metody ###################
  def self.name_merge_score_and_reason(nazev1,nazev2)
    nazev1 = nazev1.to_s.mb_chars.upcase; nazev2 = nazev2.to_s.mb_chars.upcase

    # standardní náhrady
    nazev1 = self.expand_nazev_abbrevs(nazev1); nazev2 = self.expand_nazev_abbrevs(nazev2)
    
    # nejdříve porovná nenormalizované názvy
    result = self.name_merge_score_and_reason_pom(nazev1,nazev2)
    return result if result[0] > 0

    # pokud to nestačí podívá se na normalizované verze
    result = self.name_merge_score_and_reason_pom(nazev1.normalize,nazev2.normalize)
    return result if result[0] == 0
    
    # vrátí result se sníženým skóre
    return result[0]-25, result[1] + ' v normalizované podobě'
  end
  
  # určí blízkost dvou názvů - NEvolat přímo, používá se v name_merge_score_and_reason
  def self.name_merge_score_and_reason_pom(nazev1,nazev2)
    if nazev1==nazev2
      return 80, 'jsou identické'
    elsif nazev1.start_with?(nazev2) or nazev2.start_with?(nazev1)
      return 60, 'jsou podobné - jeden z názvů je prefixem toho druhého'
    else
      case Text::Levenshtein.distance(nazev1,nazev2)
      when 0 then return 80, 'mají levenshteinovu vzdálenost je nula'
      when 1 then return 60, 'mají levenshteinovu vzdálenost 1'
      when 2 then return 40, 'mají levenshteinovu vzdálenost 2'
      end
    end
    return 0, 'názvy si nejsou podobné'
  end
  
  # zjistit zda je dobre zformatovano  a jeste volne (zatím bereme jen jasné číselné reference)
  def self.bad_kctref?(kctref)
    kctref.nil? or (!kctref.match(/^[A-Z][A-Z]\d\d\d$/))
  end
  
  def self.expand_nazev_abbrevs(nazev)
    { /ŽST/ => 'ŽELEZNIČNÍ STANICE' }.each do |key, value|
      nazev.gsub!(key, value)
    end
    return nazev
  end

  private
  def self.category_cleanup(category)
    ProposalReason.where(category: category).delete_all
  end
  
  def self.linkify(text)
    ProposalAction.linkify(text)
  end
  
end
    
    